@@ -1,61 +1,63 @@
 extern crate nalgebra as na;
 use na::{
     base::allocator::Allocator, DefaultAllocator, DimMin, DimName, DimNameAdd, DimSub, MatrixN,
-    VectorN, U1,
+    RealField, VectorN, U1,
 };
+use num_traits::Float;
 
-pub fn karmarkar<D>(
-    c: &VectorN<f64, D>,
-    amat: &MatrixN<f64, D>,
-    b: &VectorN<f64, D>,
-    x: &VectorN<f64, D>,
-    gamma: f64,
-    eps: f64,
+/// max c.T * x, subj A * x < b
+pub fn karmarkar<F, D>(
+    c: &VectorN<F, D>,
+    amat: &MatrixN<F, D>,
+    b: &VectorN<F, D>,
+    x: &VectorN<F, D>,
+    gamma: F,
+    eps: F,
     nloop: usize,
-) -> Result<VectorN<f64, D>, &'static str>
+) -> Result<VectorN<F, D>, &'static str>
 where
+    F: RealField + Float,
     D: DimName + DimNameAdd<D> + DimMin<D>,
     <D as DimMin<D>>::Output: DimSub<U1>,
-    DefaultAllocator: Allocator<f64, D>
-        + Allocator<f64, D, D>
-        + Allocator<f64, <D as DimMin<D>>::Output>
-        + Allocator<f64, <D as DimMin<D>>::Output, D>
-        + Allocator<f64, D, <D as DimMin<D>>::Output>
-        + Allocator<f64, <<D as DimMin<D>>::Output as DimSub<U1>>::Output>
-        + Allocator<f64, <D as DimMin<D>>::Output, <D as DimMin<D>>::Output>,
+    DefaultAllocator: Allocator<F, D>
+        + Allocator<F, D, D>
+        + Allocator<F, <D as DimMin<D>>::Output>
+        + Allocator<F, <D as DimMin<D>>::Output, D>
+        + Allocator<F, D, <D as DimMin<D>>::Output>
+        + Allocator<F, <<D as DimMin<D>>::Output as DimSub<U1>>::Output>
+        + Allocator<F, <D as DimMin<D>>::Output, <D as DimMin<D>>::Output>,
 {
     let mut ans = x.clone();
+    let tol = F::from_f32(1.0e-9).ok_or("Fail to convert f32 to F.")?;
     for _ in 0..nloop {
         let vk = b - amat * &ans;
-        let mut vk2 = VectorN::<f64, D>::zeros();
-        vk2.cmpy(1.0, &vk, &vk, 0.0);
-        let ivk2 = MatrixN::<f64, D>::from_diagonal(&vk2)
-            .try_inverse()
-            .ok_or("Not found inverse.")?;
+        let mut vk2 = VectorN::<F, D>::zeros();
+        vk2.cmpy(F::one(), &vk, &vk, F::zero());
+        let ivk2 = MatrixN::<F, D>::from_diagonal(&VectorN::<F, D>::identity().component_div(&vk2));
         let gmat = amat.transpose() * ivk2 * amat;
-        let pgmat = gmat.pseudo_inverse(1.0e-9)?;
+        let pgmat = gmat.pseudo_inverse(tol)?;
         let d = pgmat * c;
         if d.norm() < eps {
             break;
         }
         let hv = -amat * &d;
-        if hv.amax() <= 0.0 {
+        if hv.amax() <= F::zero() {
             return Err("Unbounded!");
         }
         let sa = (0..hv.nrows())
             .filter_map(|i| {
-                if hv[i] > 0.0 {
+                if hv[i] > F::zero() {
                     Some(vk[i] / hv[i])
                 } else {
                     None
                 }
             })
-            .fold(0.0 / 0.0, |m, v| v.min(m));
-        if sa.is_nan() {
+            .fold(Float::infinity(), |m, v| Float::min(v, m));
+        if !sa.is_finite() {
             break;
         }
         let alpha = gamma * sa;
-        ans -= alpha * &d;
+        ans -= d * alpha;
     }
     Ok(ans)
 }
